#!/usr/bin/perl
#
#  Copyright (c) 2003 Steve Slaven, All Rights Reserved.
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
#  MA 02111-1307 USA
#

use strict;
use Getopt::Std;
use Cwd;

my %o;
getopts( 'hqi:s:t:e:', \%o );

my @params;
push( @params, '--param', 'image-prefix', "'$o{ i }'" ) if $o{ i };
push( @params, '--param', 'style-prefix', "'$o{ s }'" ) if $o{ s };
push( @params, '--param', 'toc-class', "'$o{ t }'" ) if $o{ t };

my $dest = cwd();

my $input = shift() || usage();
$input = cwd() . "/$input" unless $input =~ m!^/!;
dprint( "Input file is '$input'" );

# First setup temp space
my $bin = $0;
$bin =~ s!^.*/!!;

my $tmp = "/tmp/$bin-$$";
dprint( "Temp dir id $tmp" );
mkdir $tmp || dieup( "Unable to create temp dir '$tmp'" );
chdir( $tmp );

# Now unzip original file
dprint( "Unzipping archive" );
system( "unzip", "-o", "-q", $input );

# Try and determine encoding type
open( IN, "content.xml" );
my $encoding = <IN>;
close( IN );
( $encoding ) = $encoding =~ /encoding="(.*)"/i;
$o{ e } ||= $encoding;
push( @params, "--param", "output-encoding", "'$o{e}'" );
dprint( "Output encoding: $o{e}" );

# Create html base doc
dprint( "Applying stylesheet" );
open( XSL, ">convert.xslt" );
print XSL <DATA>;
close( XSL );
system( "xsltproc", @params, "--novalid", "--output", "content.html", "convert.xslt", "content.xml" );

# Create jpg images
dprint( "Converting images" );
mkdir "image" || dieup( "Can't make image dir" );
for( glob( "Pictures/*" ) ) {
    next unless -f $_;
    my $output = $_;
    $output =~ s!^Pictures!\Qimage\E!;
    $output =~ s!\..*?$!.jpg!;
    system( "convert", $_, $output );
}

# Delete pre-existing
dprint( "Deleting files" );
unlink( "$dest/content.html" ) if -f "$dest/content.html";
unlink( glob( "$dest/image/*" ) ) if -d "$dest/image";
rmdir( "$dest/image" ) if -d "$dest/image";

# Move to dest, use system commands because it handles moving across filesystems
dprint( "Moving files" );
system( "mv", "image", $dest );
system( "mv", "content.html", $dest );

# Done!
cleanup();

sub dieup {
    cleanup();
    die( shift() );
}

sub cleanup {
    dprint( "Deleting temp dir" );
    system( "rm", "-rf", $tmp ) if -d $tmp;
}

sub dprint {
    print shift() . "\n" unless $o{ q };
}

sub usage {
    die( qq{Convert StarOffice to HTML v0.73
Author: Steve Slaven - http://hoopajoo.net
Usage: $0 [-hq] [-e encoding] [-i imagedir]
	  [-s style_base] [-t toc_class] FILE

-h	This help
-q	Quiet mode
-i	Image directory (default image/)
-s	CSS base style that all content is wrapped in (default soffice)
-t	Class name to build TOC from (default none)
-e	Output encoding, it will now try to autodetect it but you can
	override it with this switch (e.g. -e iso-8859-2)
Converts the FILE to content.html with no standard body wrapper so it can
be inserted in to existing templates.  All images are converted to JPG
and all styles are converted to CSS included in the content.html.  Requires
ImageMagick's 'convert', and 'xsltproc' from libxml.

If you use -i, it will be reflected in the html file but you will need to
rename the image directory, mostly because I felt that it was unsafe in
the case that -i was "." and could delete stuff it shouldn't
} );
}

# Put style sheet here
__DATA__
<?xml version="1.0"?>

<!-- soffice2html.xsl v0.73 -->
<!-- Converts the content.xml from an openoffice.org doc to html -->
<!-- Author: Steve Slaven - http://hoopajoo.net -->
<!-- Requires something external to handle image conversion -->
<xsl:stylesheet version="1.0"
      xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
      xmlns:fo="http://www.w3.org/1999/XSL/Format"
      xmlns:office="http://openoffice.org/2000/office"
      xmlns:style="http://openoffice.org/2000/style"
      xmlns:text="http://openoffice.org/2000/text"
      xmlns:table="http://openoffice.org/2000/table"
      xmlns:draw="http://openoffice.org/2000/drawing"
      xmlns:xlink="http://www.w3.org/1999/xlink"
      xmlns:number="http://openoffice.org/2000/datastyle"
      xmlns:svg="http://www.w3.org/2000/svg"
      xmlns:chart="http://openoffice.org/2000/chart"
      xmlns:dr3d="http://openoffice.org/2000/dr3d"
      xmlns:math="http://www.w3.org/1998/Math/MathML"
      xmlns:form="http://openoffice.org/2000/form"
      xmlns:script="http://openoffice.org/2000/script"
      xmlns:saxon="http://icl.com/saxon"
      extension-element-prefixes="saxon" >

<xsl:param name="image-prefix" select="'image'"/>
<xsl:param name="style-prefix" select="'soffice'"/>

<xsl:param name="define-base-style" select="'yes'"/>

<!-- these are really less helpful than they seem, since according to the spec an
     xslt processor is *not* required to use the same id()s every time, so in the
     future using 2 runs to make the toc and content sections could produce different
     id()s between the hrefs and names -->
<xsl:param name="toc-section" select="'yes'"/>
<xsl:param name="style-section" select="'yes'"/>
<xsl:param name="content-section" select="'yes'"/>

<xsl:param name="output-encoding" select="'utf-8'"/>

<xsl:output method="html" encoding="$output-encoding"/>

<!-- Make general document, this is the master format for the overall doc struct -->
<xsl:template match="/">
<xsl:if test="$style-section = 'yes'">
<style type="text/css">
<xsl:if test="$define-base-style = 'yes'">
.<xsl:value-of select="$style-prefix"/> {

}
</xsl:if>
.<xsl:value-of select="$style-prefix"/> ul {
  padding-left: 2ex
}
.<xsl:value-of select="$style-prefix"/> ol {
  padding-left: 2ex
}
<xsl:apply-templates select="//style:style" mode="stylesheet"/>
</style>
</xsl:if>
<xsl:apply-templates match="//office:body"/>
</xsl:template>

<!-- Generates the style sheet infos -->
<xsl:template match="style:style" mode="stylesheet">
<xsl:call-template name="generate-style">
 <xsl:with-param name="stylename" select="@style:name"/></xsl:call-template>
<xsl:call-template name="generate-style">
 <xsl:with-param name="stylename" select="@style:list-style-name"/></xsl:call-template>
</xsl:template>

<xsl:template match="style:properties" mode="stylesheet">
<xsl:call-template name="pass-css"><xsl:with-param name="attrib" select="'@style:font-name'"/></xsl:call-template>
<xsl:call-template name="pass-css"><xsl:with-param name="attrib" select="'@style:horizontal-pos'"/>
<xsl:with-param name="css" select="'text-align'"/>
<xsl:with-param name="after" select="'-'"/>
<xsl:with-param name="xlate-align" select="'yes'"/></xsl:call-template>
<xsl:call-template name="pass-css"><xsl:with-param name="attrib" select="'@fo:text-align'"/>
<xsl:with-param name="xlate-align" select="'yes'"/></xsl:call-template>
<xsl:call-template name="pass-css"><xsl:with-param name="attrib" select="'@fo:font-size'"/></xsl:call-template>
<xsl:call-template name="pass-css"><xsl:with-param name="attrib" select="'@fo:line-height'"/></xsl:call-template>
<xsl:call-template name="pass-css"><xsl:with-param name="attrib" select="'@fo:font-weight'"/></xsl:call-template>
<xsl:call-template name="pass-css"><xsl:with-param name="attrib" select="'@fo:font-style'"/></xsl:call-template>
<xsl:call-template name="pass-css"><xsl:with-param name="attrib" select="'@fo:color'"/></xsl:call-template>
<xsl:call-template name="pass-css"><xsl:with-param name="attrib" select="'@fo:padding'"/></xsl:call-template>
<xsl:call-template name="pass-css"><xsl:with-param name="attrib" select="'@fo:border'"/></xsl:call-template>
<xsl:call-template name="pass-css"><xsl:with-param name="attrib" select="'@fo:border-left'"/></xsl:call-template>
<xsl:call-template name="pass-css"><xsl:with-param name="attrib" select="'@fo:border-top'"/></xsl:call-template>
<xsl:call-template name="pass-css"><xsl:with-param name="attrib" select="'@fo:border-right'"/></xsl:call-template>
<xsl:call-template name="pass-css"><xsl:with-param name="attrib" select="'@fo:border-bottom'"/></xsl:call-template>
<xsl:call-template name="pass-css"><xsl:with-param name="attrib" select="'@style:text-background-color'"/>
<xsl:with-param name="css" select="'background'"/></xsl:call-template>
<xsl:call-template name="pass-css"><xsl:with-param name="attrib" select="'@style:width'"/></xsl:call-template>
</xsl:template>

<xsl:template name="pass-css">
<xsl:param name="attrib"/>
<xsl:param name="css" select="substring-after($attrib,':')"/>

<xsl:param name="before"/>
<xsl:param name="after"/>
<xsl:param name="output" select="saxon:evaluate($attrib)"/>

<xsl:param name="xlate-align"/>

<xsl:variable name="modified-output">
<xsl:choose>
 <xsl:when test="$before">
  <xsl:value-of select="substring-before($output,$before)"/>
 </xsl:when>
 <xsl:when test="$after">
  <xsl:value-of select="substring-after($output,$after)"/>
 </xsl:when>
 <xsl:otherwise>
  <xsl:value-of select="$output"/>
 </xsl:otherwise>
</xsl:choose>
</xsl:variable>

<xsl:variable name="final-output">
<xsl:choose>
 <xsl:when test="$xlate-align">
  <xsl:choose>
   <xsl:when test="$modified-output = 'end'">
    <xsl:value-of select="'right'"/>
   </xsl:when>
   <xsl:otherwise>
    <xsl:value-of select="$modified-output"/>
   </xsl:otherwise>
  </xsl:choose>
 </xsl:when>
 <xsl:otherwise>
  <xsl:value-of select="$modified-output"/>
 </xsl:otherwise>
</xsl:choose>
</xsl:variable>

<!-- eval even though we have a value cause it might get mangled and we're testing the eval value, not
     the passed value -->
<xsl:if test="saxon:evaluate($attrib)">  <xsl:value-of select="$css"/>: <xsl:value-of select="$final-output"/> ;
</xsl:if>
</xsl:template>

<!-- Basically a wrapper to the recursive code to make a TOC that adds the OL around it -->
<xsl:template name="generate-toc">
<xsl:param name="nodes" select="//text:h[@text:level=1]"/>
<ol><xsl:apply-templates select="$nodes" mode="toc"/></ol>
</xsl:template>

<!-- Generate toc nodes if right style type, strip off the leading style type then call itself again -->
<xsl:template match="text:h" mode="toc">
 <li><a href="#{generate-id()}"><xsl:value-of select="."/></a></li>

 <xsl:variable name="level" select="@text:level"/>
 <xsl:variable name="position" select="position()"/>
 <xsl:variable name="next-position" select="//text:h[@text:level=$level][$position + 1]"/>

 <!-- Slice of the next level, get all peer nodes between this level and the next sibling on same level -->
 <xsl:variable name="wouldbe" select="//text:h[@text:level=($level+1) and
  generate-id(following-sibling::text:h[1])=generate-id($next-position)]"/>
<xsl:if test="$wouldbe">
 <xsl:call-template name="generate-toc">
      <xsl:with-param name="nodes" select="$wouldbe"/>
     </xsl:call-template>
</xsl:if>

</xsl:template>

<!-- Main html handling -->
<xsl:template match="office:body">
<div><xsl:attribute name="class"><xsl:value-of select="$style-prefix"/></xsl:attribute>
<xsl:if test="$toc-section = 'yes'"><xsl:call-template name="generate-toc"/>
</xsl:if><xsl:if test="$content-section = 'yes'">
<xsl:apply-templates/></xsl:if>
</div>
</xsl:template>

<xsl:template match="text:p">
<p><xsl:call-template name="insert-style"/>
<xsl:apply-templates/></p>
</xsl:template>

<xsl:template match="text:span">
<span><xsl:call-template name="insert-style"/><xsl:apply-templates/></span>
</xsl:template>

<xsl:template match="text:tab-stop">
<span style="padding: 2ex"></span>
</xsl:template>

<xsl:template match="text:s">
<xsl:call-template name="nbsp"/>
</xsl:template>

<xsl:template match="text:h">
<div><xsl:call-template name="insert-style"/>
<a name="{generate-id()}"/><xsl:apply-templates/></div>
</xsl:template>

<xsl:template match="text:unordered-list">
<ul><xsl:call-template name="insert-style"/><xsl:apply-templates/></ul>
</xsl:template>

<xsl:template match="text:list-item">
<li><xsl:call-template name="insert-style"/><xsl:apply-templates/></li>
</xsl:template>

<!-- This presumes some external script will handle image conversion -->
<xsl:template match="draw:image">

<!-- handle internal/external href, if the hrefs are external they will have the
     prefix prepended to them and the extension will be changed to .jpg, it's up
     to some kind of wrapper to convert the images there -->
<xsl:variable name="final-uri">
  <xsl:choose>
  <xsl:when test="substring-before(@xlink:href,1)='#'">
    <xsl:value-of select="substring-before(substring(@xlink:href,11),'.')"/>.jpg</xsl:when>
  <xsl:otherwise>
    <xsl:value-of select="substring-before(@xlink:href,'.')"/>.jpg</xsl:otherwise>
  </xsl:choose>
</xsl:variable>

<span><xsl:call-template name="insert-style"/>
<img><xsl:attribute name="src">
<xsl:value-of select="$image-prefix"/>/<xsl:value-of select="$final-uri"/></xsl:attribute>
</img></span><br/>
</xsl:template>

<xsl:template match="draw:text-box">
<div><xsl:call-template name="insert-style"/><xsl:apply-templates/></div>
</xsl:template>

<xsl:template match="table:table">
<table><xsl:call-template name="insert-style"/><xsl:apply-templates/></table>
</xsl:template>

<xsl:template match="table:table-header-rows">
<xsl:apply-templates/>
</xsl:template>

<xsl:template match="table:table-row">
<tr><xsl:call-template name="insert-style"/><xsl:apply-templates/></tr>
</xsl:template>

<xsl:template match="table:table-cell">
<td><xsl:call-template name="insert-style"/><xsl:apply-templates/></td>
</xsl:template>

<!-- call by name stuff -->
<xsl:template name="insert-style">
 <xsl:call-template name="insert-style-work">
  <xsl:with-param name="stylename" select="@text:style-name"/>
 </xsl:call-template>

 <xsl:call-template name="insert-style-work">
  <xsl:with-param name="stylename" select="@draw:style-name"/>
 </xsl:call-template>

 <xsl:call-template name="insert-style-work">
  <xsl:with-param name="stylename" select="@table:style-name"/>
 </xsl:call-template>
</xsl:template>

<xsl:template name="insert-style-work">
 <xsl:param name="stylename"/>
 <xsl:param name="final-stylename" select="translate($stylename,'. ','_')"/>

 <xsl:if test="$stylename != ''">
  <xsl:attribute name="class">
    <xsl:value-of select="$style-prefix"/>_<xsl:value-of select="$final-stylename"/></xsl:attribute>
 </xsl:if>
</xsl:template>

<xsl:template name="generate-style">
 <xsl:param name="stylename"/>
 <!-- cleanse name -->
 <xsl:param name="final-stylename" select="translate($stylename,'. ','_')"/>

  <xsl:if test="$stylename">
.<xsl:value-of select="$style-prefix"/> .<xsl:value-of
   select="$style-prefix"/>_<xsl:value-of select="$final-stylename"/> {
<xsl:apply-templates mode="stylesheet"/>}
</xsl:if>
</xsl:template>

<!-- wow this sucks -->
<xsl:template name="nbsp">
<xsl:text disable-output-escaping="yes">&amp;nbsp;</xsl:text>
</xsl:template>

</xsl:stylesheet>
